"""
Two categories of input:
    - Single Item
        - A single file
        - A stream of text characters
    - Multiple items
        - A list of files


1. Is it a single item?
    - If yes, attempt to parse.
        - If success, pass to Result.Counted
        - If failed, pass to Result.Not Counted
2. If no, spawn threads equivalent to machine capacity:
    2a. Perform Step 1.
"""
